@@ -17,78 +17,19 @@
 
 ;;;; the CLASSOID structure
 
-;;; The CLASSOID structure is a supertype of all classoid types.  A
-;;; CLASSOID is also a CTYPE structure as recognized by the type
-;;; system.  (FIXME: It's also a type specifier, though this might go
-;;; away as with the merger of SB-PCL:CLASS and CL:CLASS it's no
-;;; longer necessary)
-(def!struct (classoid
-             (:make-load-form-fun classoid-make-load-form-fun)
-             (:include ctype
-                       (class-info (type-class-or-lose 'classoid)))
-             (:constructor nil)
-             #-no-ansi-print-object
-             (:print-object
-              (lambda (class stream)
-                (let ((name (classoid-name class)))
-                  (print-unreadable-object (class stream
-                                                  :type t
-                                                  :identity (not name))
-                    (format stream
-                            ;; FIXME: Make sure that this prints
-                            ;; reasonably for anonymous classes.
-                            "~:[anonymous~;~:*~S~]~@[ (~(~A~))~]"
-                            name
-                            (classoid-state class))))))
-             #-sb-xc-host (:pure nil))
-  ;; the value to be returned by CLASSOID-NAME.
-  (name nil :type symbol)
-  ;; the current layout for this class, or NIL if none assigned yet
-  (layout nil :type (or layout null))
-  ;; How sure are we that this class won't be redefined?
-  ;;   :READ-ONLY = We are committed to not changing the effective
-  ;;                slots or superclasses.
-  ;;   :SEALED    = We can't even add subclasses.
-  ;;   NIL        = Anything could happen.
-  (state nil :type (member nil :read-only :sealed))
-  ;; direct superclasses of this class. Always NIL for CLOS classes.
-  (direct-superclasses () :type list)
-  ;; representation of all of the subclasses (direct or indirect) of
-  ;; this class. This is NIL if no subclasses or not initalized yet;
-  ;; otherwise, it's an EQ hash-table mapping CLASSOID objects to the
-  ;; subclass layout that was in effect at the time the subclass was
-  ;; created.
-  (subclasses nil :type (or null hash-table))
-  ;; the PCL class (= CL:CLASS, but with a view to future flexibility
-  ;; we don't just call it the CLASS slot) object for this class, or
-  ;; NIL if none assigned yet
-  (pcl-class nil))
+;;; The CLASSOID structure is a supertype of all classoid types.
+;;; Its definition occurs in 'early-classoid.lisp'
+
+(defun classoid-make-load-form-fun (self)
+  (let ((name (classoid-name self)))
+    (if (and name (eq (find-classoid name nil) self))
+        `(find-classoid ',name)
+        (error "can't use anonymous or undefined class as constant:~%  ~S"
+               self))))
 
-(defun classoid-make-load-form-fun (class)
-  (/show "entering CLASSOID-MAKE-LOAD-FORM-FUN" class)
-  (let ((name (classoid-name class)))
-    (unless (and name (eq (find-classoid name nil) class))
-      (/show "anonymous/undefined class case")
-      (error "can't use anonymous or undefined class as constant:~%  ~S"
-             class))
-    `(locally
-       ;; KLUDGE: There's a FIND-CLASSOID DEFTRANSFORM for constant
-       ;; class names which creates fast but non-cold-loadable,
-       ;; non-compact code. In this context, we'd rather have compact,
-       ;; cold-loadable code. -- WHN 19990928
-       (declare (notinline find-classoid))
-       (find-classoid ',name))))
 
 ;;;; basic LAYOUT stuff
 
-;;; Note: This bound is set somewhat less than MOST-POSITIVE-FIXNUM
-;;; in order to guarantee that several hash values can be added without
-;;; overflowing into a bignum.
-(def!constant layout-clos-hash-limit (1+ (ash sb!xc:most-positive-fixnum -3))
-  #!+sb-doc
-  "the exclusive upper bound on LAYOUT-CLOS-HASH values")
-(def!type layout-clos-hash () '(integer 0 #.layout-clos-hash-limit))
-
 ;;; a list of conses, initialized by genesis
 ;;;
 ;;; In each cons, the car is the symbol naming the layout, and the
@@ -111,114 +52,7 @@
                          (cdr x)))
                  (/show0 "done processing *!INITIAL-LAYOUTS*")))
 
-;;; The LAYOUT structure is pointed to by the first cell of instance
-;;; (or structure) objects. It represents what we need to know for
-;;; type checking and garbage collection. Whenever a class is
-;;; incompatibly redefined, a new layout is allocated. If two object's
-;;; layouts are EQ, then they are exactly the same type.
-;;;
-;;; *** IMPORTANT ***
-;;;
-;;; If you change the slots of LAYOUT, you need to alter genesis as
-;;; well, since the initialization of layout slots is hardcoded there.
-;;;
-;;; FIXME: ...it would be better to automate this, of course...
-(def!struct (layout
-             ;; KLUDGE: A special hack keeps this from being
-             ;; called when building code for the
-             ;; cross-compiler. See comments at the DEFUN for
-             ;; this. -- WHN 19990914
-             (:make-load-form-fun #-sb-xc-host ignore-it
-                                  ;; KLUDGE: DEF!STRUCT at #+SB-XC-HOST
-                                  ;; time controls both the
-                                  ;; build-the-cross-compiler behavior
-                                  ;; and the run-the-cross-compiler
-                                  ;; behavior. The value below only
-                                  ;; works for build-the-cross-compiler.
-                                  ;; There's a special hack in
-                                  ;; EMIT-MAKE-LOAD-FORM which gives
-                                  ;; effectively IGNORE-IT behavior for
-                                  ;; LAYOUT at run-the-cross-compiler
-                                  ;; time. It would be cleaner to
-                                  ;; actually have an IGNORE-IT value
-                                  ;; stored, but it's hard to see how to
-                                  ;; do that concisely with the current
-                                  ;; DEF!STRUCT setup. -- WHN 19990930
-                                  #+sb-xc-host
-                                  make-load-form-for-layout))
-  ;; a pseudo-random hash value for use by CLOS.  KLUDGE: The fact
-  ;; that this slot is at offset 1 is known to GENESIS.
-  (clos-hash (random-layout-clos-hash) :type layout-clos-hash)
-  ;; the class that this is a layout for
-  (classoid (missing-arg) :type classoid)
-  ;; The value of this slot can be:
-  ;;   * :UNINITIALIZED if not initialized yet;
-  ;;   * NIL if this is the up-to-date layout for a class; or
-  ;;   * T if this layout has been invalidated (by being replaced by
-  ;;     a new, more-up-to-date LAYOUT).
-  ;;   * something else (probably a list) if the class is a PCL wrapper
-  ;;     and PCL has made it invalid and made a note to itself about it
-  (invalid :uninitialized :type (or cons (member nil t :uninitialized)))
-  ;; the layouts for all classes we inherit. If hierarchical, i.e. if
-  ;; DEPTHOID >= 0, then these are ordered by ORDER-LAYOUT-INHERITS
-  ;; (least to most specific), so that each inherited layout appears
-  ;; at its expected depth, i.e. at its LAYOUT-DEPTHOID value.
-  ;;
-  ;; Remaining elements are filled by the non-hierarchical layouts or,
-  ;; if they would otherwise be empty, by copies of succeeding layouts.
-  (inherits #() :type simple-vector)
-  ;; If inheritance is not hierarchical, this is -1. If inheritance is
-  ;; hierarchical, this is the inheritance depth, i.e. (LENGTH INHERITS).
-  ;; Note:
-  ;;  (1) This turns out to be a handy encoding for arithmetically
-  ;;      comparing deepness; it is generally useful to do a bare numeric
-  ;;      comparison of these depthoid values, and we hardly ever need to
-  ;;      test whether the values are negative or not.
-  ;;  (2) This was called INHERITANCE-DEPTH in classic CMU CL. It was
-  ;;      renamed because some of us find it confusing to call something
-  ;;      a depth when it isn't quite.
-  (depthoid -1 :type layout-depthoid)
-  ;; the number of top level descriptor cells in each instance
-  (length 0 :type index)
-  ;; If this layout has some kind of compiler meta-info, then this is
-  ;; it. If a structure, then we store the DEFSTRUCT-DESCRIPTION here.
-  (info nil :type (or null defstruct-description))
-  ;; This is true if objects of this class are never modified to
-  ;; contain dynamic pointers in their slots or constant-like
-  ;; substructure (and hence can be copied into read-only space by
-  ;; PURIFY).
-  ;;
-  ;; This slot is known to the C runtime support code.
-  (pure nil :type (member t nil 0))
-  ;; Number of raw words at the end.
-  ;; This slot is known to the C runtime support code.
-  ;; It counts the number of untagged cells, not user-visible slots.
-  ;; e.g. on 32-bit machines, each (COMPLEX DOUBLE-FLOAT) counts as 4.
-  #!-interleaved-raw-slots (n-untagged-slots 0 :type index)
-  ;; Metadata
-  #!+interleaved-raw-slots (untagged-bitmap 0 :type unsigned-byte)
-  #!+interleaved-raw-slots (equalp-tests #() :type simple-vector)
-  ;; Definition location
-  (source-location nil)
-  ;; If this layout is for an object of metatype STANDARD-CLASS,
-  ;; these are the EFFECTIVE-SLOT-DEFINITION metaobjects.
-  (slot-list nil :type list)
-  ;; Information about slots in the class to PCL: this provides fast
-  ;; access to slot-definitions and locations by name, etc.
-  ;; See MAKE-SLOT-TABLE in pcl/slots-boot.lisp for further details.
-  (slot-table #(1 nil) :type simple-vector)
-  ;; True IFF the layout belongs to a standand-instance or a
-  ;; standard-funcallable-instance.
-  ;; Old comment was:
-  ;;   FIXME: If we unify wrappers and layouts this can go away, since
-  ;;   it is only used in SB-PCL::EMIT-FETCH-WRAPPERS, which can then
-  ;;   use INSTANCE-SLOTS-LAYOUT instead (if there is are no slot
-  ;;   layouts, there are no slots for it to pull.)
-  ;; But while that's conceivable, it still seems advantageous to have
-  ;; a single bit that decides whether something is STANDARD-OBJECT.
-  (%for-std-class-b 0 :type bit :read-only t))
-
-(declaim (freeze-type layout)) ; Good luck hot-patching new subtypes of LAYOUT
+;;; The LAYOUT structure itself is defined in 'early-classoid.lisp'
 
 (declaim (inline layout-for-std-class-p))
 (defun layout-for-std-class-p (x) (not (zerop (layout-%for-std-class-b x))))
@@ -461,7 +295,9 @@ between the ~A definition and the ~A definition"
                           layout)
                 find-and-init-or-check-layout))
 (defun find-and-init-or-check-layout (name length inherits depthoid metadata)
-  (with-world-lock ()
+  (truly-the ; avoid an "assertion too complex to check" optimizer note
+   (values layout &optional)
+   (with-world-lock ()
     (let ((layout (find-layout name)))
       (%init-or-check-layout layout
                              (or (find-classoid name nil)
@@ -469,7 +305,7 @@ between the ~A definition and the ~A definition"
                              length
                              inherits
                              depthoid
-                             metadata))))
+                             metadata)))))
 
 ;;; Record LAYOUT as the layout for its class, adding it as a subtype
 ;;; of all superclasses. This is the operation that "installs" a
@@ -670,28 +506,10 @@ between the ~A definition and the ~A definition"
 
 ;;;; object types to represent classes
 
-;;; An UNDEFINED-CLASSOID is a cookie we make up to stick in forward
-;;; referenced layouts. Users should never see them.
-(def!struct (undefined-classoid
-             (:include classoid)
-             (:constructor make-undefined-classoid (name))))
-
 ;;; BUILT-IN-CLASS is used to represent the standard classes that
 ;;; aren't defined with DEFSTRUCT and other specially implemented
 ;;; primitive types whose only attribute is their name.
-;;;
-;;; Some BUILT-IN-CLASSes have a TRANSLATION, which means that they
-;;; are effectively DEFTYPE'd to some other type (usually a union of
-;;; other classes or a "primitive" type such as NUMBER, ARRAY, etc.)
-;;; This translation is done when type specifiers are parsed. Type
-;;; system operations (union, subtypep, etc.) should never encounter
-;;; translated classes, only their translation.
-(def!struct (built-in-classoid (:include classoid)
-                               (:constructor make-built-in-classoid))
-  ;; the type we translate to on parsing. If NIL, then this class
-  ;; stands on its own; or it can be set to :INITIALIZING for a period
-  ;; during cold-load.
-  (translation nil :type (or ctype (member nil :initializing))))
+;;; It is defined in 'early-classoid.lisp'
 
 ;;; STRUCTURE-CLASS represents what we need to know about structure
 ;;; classes. Non-structure "typed" defstructs are a special case, and
@@ -703,48 +521,7 @@ between the ~A definition and the ~A definition"
 
 ;;;; classoid namespace
 
-;;; We use an indirection to allow forward referencing of class
-;;; definitions with load-time resolution.
-(def!struct (classoid-cell
-             (:constructor make-classoid-cell (name &optional classoid))
-             (:make-load-form-fun (lambda (c)
-                                    `(find-classoid-cell
-                                      ',(classoid-cell-name c)
-                                      :create t)))
-             #-no-ansi-print-object
-             (:print-object (lambda (s stream)
-                              (print-unreadable-object (s stream :type t)
-                                (prin1 (classoid-cell-name s) stream)))))
-  ;; Name of class we expect to find.
-  (name nil :type symbol :read-only t)
-  ;; Classoid or NIL if not yet defined.
-  (classoid nil :type (or classoid null))
-  ;; PCL class, if any
-  (pcl-class nil))
-(declaim (freeze-type classoid-cell))
-
-(defun find-classoid-cell (name &key create)
-  (let ((real-name (uncross name)))
-    (cond ((info :type :classoid-cell real-name))
-          (create
-           (get-info-value-initializing :type :classoid-cell real-name
-                                        (make-classoid-cell real-name))))))
-
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-
-  ;; Return the classoid with the specified NAME. If ERRORP is false,
-  ;; then NIL is returned when no such class exists."
-  (defun find-classoid (name &optional (errorp t))
-    (declare (type symbol name))
-    (let ((cell (find-classoid-cell name)))
-      (cond ((and cell (classoid-cell-classoid cell)))
-            (errorp
-             (error 'simple-type-error
-                    :datum nil
-                    :expected-type 'class
-                    :format-control "Class not yet defined: ~S"
-                    :format-arguments (list name))))))
-
   (defun (setf find-classoid) (new-value name)
     #-sb-xc (declare (type (or null classoid) new-value))
     (aver new-value)
@@ -871,8 +648,10 @@ between the ~A definition and the ~A definition"
 
 ;;;; CLASS type operations
 
-;; referenced right away by !DEFINE-TYPE-CLASS.
-(eval-when (:compile-toplevel :load-toplevel :execute)
+;; CLASSOID-ENUMERABLE-P is referenced during compile by !DEFINE-TYPE-CLASS.
+;; But don't redefine it when building the target since we've already
+;; got a perfectly good definition loaded for the host.
+(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
   ;; Actually this definition makes very little sense because
   ;;     (TYPE-ENUMERABLE (FIND-CLASSOID 'CHARACTER)) => T
   ;; but (TYPE-ENUMERABLE (SPECIFIER-TYPE 'CHARACTER)) => NIL.
@@ -1033,19 +812,6 @@ between the ~A definition and the ~A definition"
 (!define-type-method (classoid :unparse) (type)
   (classoid-proper-name type))
 
-;;;; PCL stuff
-
-;;; the CLASSOID that we use to represent type information for
-;;; STANDARD-CLASS and FUNCALLABLE-STANDARD-CLASS.  The type system
-;;; side does not need to distinguish between STANDARD-CLASS and
-;;; FUNCALLABLE-STANDARD-CLASS.
-(def!struct (standard-classoid (:include classoid)
-                               (:constructor make-standard-classoid)))
-;;; a metaclass for classes which aren't standardlike but will never
-;;; change either.
-(def!struct (static-classoid (:include classoid)
-                             (:constructor make-static-classoid)))
-
 ;;;; built-in classes
 
 ;;; The BUILT-IN-CLASSES list is a data structure which configures the
@@ -1263,14 +1029,14 @@ between the ~A definition and the ~A definition"
       :direct-superclasses (vector simple-array)
       :inherits (vector simple-array array sequence)
       :prototype-form (make-array 0 :element-type '(unsigned-byte 32)))
-     #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+     #!+64-bit
      (simple-array-unsigned-byte-63
       :translation (simple-array (unsigned-byte 63) (*))
       :codes (#.sb!vm:simple-array-unsigned-byte-63-widetag)
       :direct-superclasses (vector simple-array)
       :inherits (vector simple-array array sequence)
       :prototype-form (make-array 0 :element-type '(unsigned-byte 63)))
-     #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+     #!+64-bit
      (simple-array-unsigned-byte-64
       :translation (simple-array (unsigned-byte 64) (*))
       :codes (#.sb!vm:simple-array-unsigned-byte-64-widetag)
@@ -1306,7 +1072,7 @@ between the ~A definition and the ~A definition"
       :direct-superclasses (vector simple-array)
       :inherits (vector simple-array array sequence)
       :prototype-form (make-array 0 :element-type '(signed-byte 32)))
-     #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+     #!+64-bit
      (simple-array-signed-byte-64
       :translation (simple-array (signed-byte 64) (*))
       :codes (#.sb!vm:simple-array-signed-byte-64-widetag)
@@ -1586,15 +1352,7 @@ between the ~A definition and the ~A definition"
 (!cold-init-forms
   #-sb-xc-host (/show0 "about to set *BUILT-IN-CLASS-CODES*")
   (setq **built-in-class-codes**
-        (let* ((initial-element
-                (locally
-                  ;; KLUDGE: There's a FIND-CLASSOID DEFTRANSFORM for
-                  ;; constant class names which creates fast but
-                  ;; non-cold-loadable, non-compact code. In this
-                  ;; context, we'd rather have compact, cold-loadable
-                  ;; code. -- WHN 19990928
-                  (declare (notinline find-classoid))
-                  (classoid-layout (find-classoid 'random-class))))
+        (let* ((initial-element (classoid-layout (find-classoid 'random-class)))
                (res (make-array 256 :initial-element initial-element)))
           (dolist (x *!built-in-classes* res)
             (destructuring-bind (name &key codes &allow-other-keys)
