@@ -11,33 +11,27 @@
 
 (in-package "SB!IMPL")
 
-(macrolet ((def (name result access src-type &optional typep)
-             `(defun ,name (object ,@(if typep '(type) ()))
-                (declare (type ,(ecase src-type
-                                       (:list 'list)
-                                       (:vector 'vector)
-                                       (:sequence 'sequence)) object))
+(macrolet ((def (name constructor access src-type)
+             `(defun ,name (object type)
+                (declare (type ,src-type object))
                 (do* ((index 0 (1+ index))
                       (length (length object))
-                      (result ,result)
+                      (result ,constructor)
                       (in-object object))
                      ((>= index length) result)
                   (declare (fixnum length index))
                   (declare (type vector result))
                   (setf (,access result index)
                         ,(ecase src-type
-                           (:list '(pop in-object))
-                           (:vector '(aref in-object index))
-                           (:sequence '(elt in-object index))))))))
+                           (list '(pop in-object))
+                           (vector '(aref in-object index))
+                           (sequence '(elt in-object index))))))))
 
-  (def list-to-vector* (make-sequence type length)
-    aref :list t)
+  (def list-to-vector* (make-sequence type length) aref list)
 
-  (def vector-to-vector* (make-sequence type length)
-    aref :vector t)
+  (def vector-to-vector* (make-sequence type length) aref vector)
 
-  (def sequence-to-vector* (make-sequence type length)
-    aref :sequence t))
+  (def sequence-to-vector* (make-sequence type length) aref sequence))
 
 (defun vector-to-list* (object)
   (declare (type vector object))
@@ -68,26 +62,17 @@
 (declaim (inline coerce-to-list))
 (declaim (inline coerce-to-vector))
 
-(defun coerce-symbol-to-fun (object)
+(defun coerce-symbol-to-fun (symbol)
   ;; FIXME? I would think to use SYMBOL-FUNCTION here which does not strip off
   ;; encapsulations. But Stas wrote FDEFINITION so ...
   ;; [Also note, we won't encapsulate a macro or special-form, so this
   ;; introspective technique to decide what kind something is works either way]
-  (let* ((def (fdefinition object))
-         (widetag (fun-subtype def)))
-    (cond ((and (eq widetag sb!vm:closure-header-widetag)
-                (eq (%closure-fun def)
-                    (load-time-value
-                     ;; pick a macro, any macro...
-                     (%closure-fun (symbol-function 'with-unique-names))
-                     t)))
-           (error "~S names a macro." object))
-          ((and (eq widetag sb!vm:simple-fun-header-widetag)
-                (let ((name (%simple-fun-name def)))
-                  (and (listp name) (eq (car name) 'special-operator))))
-           (error "~S names a special operator." object))
-          (t
-           def))))
+  (let ((def (fdefinition symbol)))
+    (acond ((macro/special-guard-fun-p def)
+            (error (if (eq it :special)
+                       "~S names a special operator." "~S names a macro.")
+                   symbol))
+           (t def))))
 
 (defun coerce-to-fun (object)
   ;; (Unlike the other COERCE-TO-FOOs, this one isn't inline, because
@@ -135,7 +120,6 @@
   #!+sb-doc
   "Coerce the Object to an object of type Output-Type-Spec."
   (flet ((coerce-error ()
-           (/show0 "entering COERCE-ERROR")
            (error 'simple-type-error
                   :format-control "~S can't be converted to type ~S."
                   :format-arguments (list object output-type-spec)
@@ -143,7 +127,7 @@
                   :expected-type output-type-spec)))
     (let ((type (specifier-type output-type-spec)))
       (cond
-        ((%typep object output-type-spec)
+        ((%%typep object type)
          object)
         ((eq type *empty-type*)
          (coerce-error))
@@ -290,7 +274,7 @@
            result))
     (let ((type (specifier-type output-type-spec)))
       (cond
-        ((%typep object output-type-spec)
+        ((%%typep object type)
          object)
         ((eq type *empty-type*)
          (coerce-error))
