@@ -75,23 +75,43 @@
    ARGUMENTS. If the condition is not handled, NIL is returned. If
    (TYPEP condition *BREAK-ON-SIGNALS*) is true, the debugger is invoked
    before any signalling is done."
-  (let ((condition (coerce-to-condition datum
-                                        arguments
-                                        'simple-condition
-                                        'signal))
-        (handler-clusters *handler-clusters*)
-        (sb!debug:*stack-top-hint* (or sb!debug:*stack-top-hint* 'signal)))
+  (let* ((condition
+          (coerce-to-condition datum arguments 'simple-condition 'signal))
+         (handler-clusters *handler-clusters*)
+         (sb!debug:*stack-top-hint* (or sb!debug:*stack-top-hint* 'signal))
+         ;; possible FIXME: despite that fndb says COERCE-TO-CONDITION
+         ;; returns CONDITION, compiler doesn't know that CONDITION
+         ;; is an INSTANCE and emits a type-check.
+         (layout (%instance-layout condition)))
     (when *break-on-signals*
       (maybe-break-on-signal condition))
-    (loop
-      (unless handler-clusters
-        (return))
-      (let ((cluster (pop handler-clusters)))
+    (do ((cluster (pop handler-clusters) (pop handler-clusters)))
+        ((null cluster)
+         nil)
+      ;; Remove CLUSTER from *HANDLER-CLUSTERS*: if a condition is
+      ;; signaled in either the type test, i.e. (the function (car
+      ;; handler)), or the handler, (the function (cdr handler)), the
+      ;; recursive SIGNAL call should not consider CLUSTER as doing
+      ;; would lead to infinite recursive SIGNAL calls.
+      (let ((*handler-clusters* handler-clusters))
         (dolist (handler cluster)
-          (when (typep condition (car handler))
-            (let ((*handler-clusters* handler-clusters))
-              (funcall (cdr handler) condition))))))
-    nil))
+          (macrolet ((cast-to-fun (f possibly-symbolp)
+                       ;; For efficiency the cases are tested in this order:
+                       ;;  - FUNCTIONP is just a lowtag test
+                       ;;  - FDEFN-P is a lowtag + widetag.
+                       ;; Avoiding a SYMBOLP test is fine because
+                       ;; SYMBOL-FUNCTION rejects bogosity anyway.
+                       `(let ((f ,f))
+                          (cond ((functionp f) f)
+                                (,(if possibly-symbolp `(fdefn-p f) 't)
+                                  (sb!c:safe-fdefn-fun f))
+                                ,@(if possibly-symbolp
+                                      `((t (symbol-function f))))))))
+            (let ((test (car (truly-the cons handler))))
+              (when (if (%instancep test) ; a condition classoid cell
+                        (classoid-cell-typep layout test condition)
+                        (funcall (cast-to-fun test nil) condition))
+                (funcall (cast-to-fun (cdr handler) t) condition)))))))))
 
 (defun error (datum &rest arguments)
   #!+sb-doc
@@ -150,32 +170,32 @@ of condition handling occurring."
         (sb!debug:*stack-top-hint* (or sb!debug:*stack-top-hint* 'break)))
     (apply #'%break 'break datum arguments)))
 
+(defun %warn (datum arguments super default-type)
+  (infinite-error-protect
+    (let ((condition (coerce-to-condition datum arguments default-type 'warn))
+          (superclassoid-name (classoid-name super)))
+      ;: CONDITION is necessarily an INSTANCE,
+      ;; but pedantry requires it be the right subtype of instance.
+      (unless (classoid-typep (%instance-layout condition)
+                              super condition)
+        (error 'simple-type-error
+               :datum datum :expected-type superclassoid-name
+               :format-control "~S does not designate a ~A class"
+               :format-arguments (list datum superclassoid-name)))
+      (restart-case (signal condition)
+        (muffle-warning ()
+          :report "Skip warning."
+          (return-from %warn nil)))
+      (format *error-output* "~&~@<~S: ~3i~:_~A~:>~%"
+              superclassoid-name condition)))
+  nil)
+
 (defun warn (datum &rest arguments)
   #!+sb-doc
   "Warn about a situation by signalling a condition formed by DATUM and
    ARGUMENTS. While the condition is being signaled, a MUFFLE-WARNING restart
    exists that causes WARN to immediately return NIL."
-  (/show0 "entering WARN")
-  (infinite-error-protect
-       (/show0 "doing COERCE-TO-CONDITION")
-       (let ((condition (coerce-to-condition datum arguments
-                                             'simple-warning 'warn)))
-         (/show0 "back from COERCE-TO-CONDITION, doing ENFORCE-TYPE")
-         (enforce-type condition warning)
-         (/show0 "back from ENFORCE-TYPE, doing RESTART-CASE MUFFLE-WARNING")
-         (restart-case (signal condition)
-           (muffle-warning ()
-             :report "Skip warning."
-             (return-from warn nil)))
-         (/show0 "back from RESTART-CASE MUFFLE-WARNING (i.e. normal return)")
+  (%warn datum arguments (find-classoid 'warning) 'simple-warning))
 
-         (let ((badness (etypecase condition
-                          (style-warning 'style-warning)
-                          (warning 'warning))))
-           (/show0 "got BADNESS, calling FORMAT")
-           (format *error-output*
-                   "~&~@<~S: ~3i~:_~A~:>~%"
-                   badness
-                   condition)
-           (/show0 "back from FORMAT, voila!"))))
-  nil)
+(defun style-warn (datum &rest arguments)
+  (%warn datum arguments (find-classoid 'style-warning) 'simple-style-warning))
