@@ -126,14 +126,6 @@
                       (t `(values ,@(cdr result) &optional)))))
     `(function ,args ,result)))
 
-;;; a type specifier
-;;;
-;;; FIXME: The SB!KERNEL:INSTANCE here really means CL:CLASS.
-;;; However, the CL:CLASS type is only defined once PCL is loaded,
-;;; which is before this is evaluated.  Once PCL is moved into cold
-;;; init, this might be fixable.
-(def!type type-specifier () '(or list symbol instance))
-
 ;;; the default value used for initializing character data. The ANSI
 ;;; spec says this is arbitrary, so we use the value that falls
 ;;; through when we just let the low-level consing code initialize
@@ -1341,9 +1333,14 @@
 
 (defun check-deprecated-type (type-specifier)
   (typecase type-specifier
-    ((and type-specifier (not instance))
+    ((or symbol cons)
      (%check-deprecated-type type-specifier))
     (class
+     ;; FIXME: this case does not acknowledge that improperly named classes
+     ;; can exist. Suppose a few classes each have CLASS-NAME = FRED
+     ;; but (FIND-CLASS 'FRED) does not return any of them; and simultaneously
+     ;; FRED is a completely unrelated type specifier defined via DEFTYPE.
+     ;; This should see that class-name does not properly name the class.
      (let ((name (class-name type-specifier)))
        (when (and name (symbolp name))
          (%check-deprecated-type name))))))
