@@ -124,19 +124,25 @@
                                        (subseq optional 0 (1+ last-not-rest))))
                                 rest))))
 
-(defun parse-args-types (lambda-listy-thing context)
+;; CONTEXT is the cookie passed down from the outermost surrounding call
+;; of VALUES-SPECIFIER-TYPE. INNER-CONTEXT-KIND is an indicator of whether
+;; we are currently parsing a FUNCTION or a VALUES compound type specifier.
+(defun parse-args-types (context lambda-listy-thing inner-context-kind)
   (multiple-value-bind (llks required optional rest keys)
       (parse-lambda-list
        lambda-listy-thing
-       :context context
-       :accept (ecase context
+       :context inner-context-kind
+       :accept (ecase inner-context-kind
                  (:values-type (lambda-list-keyword-mask '(&optional &rest)))
                  (:function-type (lambda-list-keyword-mask
                                   '(&optional &rest &key &allow-other-keys))))
        :silent t)
-    (let ((required (mapcar #'single-value-specifier-type required))
-          (optional (mapcar #'single-value-specifier-type optional))
-          (rest (when rest (single-value-specifier-type (car rest))))
+   (flet ((parse-list (list)
+            (mapcar (lambda (x) (single-value-specifier-type-r context x))
+                    list)))
+    (let ((required (parse-list required))
+          (optional (parse-list optional))
+          (rest (when rest (single-value-specifier-type-r context (car rest))))
           (keywords
            (collect ((key-info))
              (dolist (key keys)
@@ -148,13 +154,15 @@
                           kwd lambda-listy-thing))
                  (key-info
                   (make-key-info
+                   ;; MAKE-KEY-INFO will complain if KWD is not a symbol.
+                   ;; That's good enough - we don't need an extra check here.
                    :name kwd
-                   :type (single-value-specifier-type (second key))))))
+                   :type (single-value-specifier-type-r context (second key))))))
              (key-info))))
       (multiple-value-bind (required optional rest)
           (canonicalize-args-type-args required optional rest
                                        (ll-kwds-keyp llks))
-        (values llks required optional rest keywords)))))
+        (values llks required optional rest keywords))))))
 
 (defstruct (values-type
             (:include args-type
@@ -882,94 +890,143 @@
 
 ;;;; type utilities
 
-;;; Return the type structure corresponding to a type specifier. We
-;;; pick off structure types as a special case.
+;;; Return the type structure corresponding to a type specifier.
 ;;;
 ;;; Note: VALUES-SPECIFIER-TYPE-CACHE-CLEAR must be called whenever a
 ;;; type is defined (or redefined).
-;;; This cache is sized extremely generously, which has payoff
-;;; elsewhere: it improves the TYPE= and CSUBTYPEP functions,
-;;; since EQ types are an immediate win.
 ;;;
-;;; KLUDGE: why isn't this a MACROLET?  "lexical environment too
-;;; hairy"
-(defmacro !values-specifier-type-body (arg)
-  `(let* ((u (uncross ,arg))
-          (cachep t)
-          (result (or (info :type :builtin u)
-                      (let ((spec (typexpand u)))
-                        (when (and (symbolp u) (deprecated-thing-p 'type u))
-                          (setf cachep nil)
-                          (signal 'parse-deprecated-type :specifier u))
-                        (cond
-                          ((and (not (eq spec u))
-                                (info :type :builtin spec)))
-                          ((and (consp spec) (symbolp (car spec))
-                                (info :type :builtin (car spec))
-                                (let ((expander (info :type :expander (car spec))))
-                                  (and expander (values-specifier-type (funcall expander spec))))))
-                          ((eq (info :type :kind spec) :instance)
-                           (find-classoid spec))
-                          ((typep spec 'classoid)
-                           (if (typep spec 'built-in-classoid)
-                               (or (built-in-classoid-translation spec) spec)
-                               spec))
-                          (t
-                           (when (and (atom spec)
-                                      (member spec '(and or not member eql satisfies values)))
-                             (error "The symbol ~S is not valid as a type specifier." spec))
-                           (let ((fun-or-ctype
-                                  (info :type :translator (if (consp spec) (car spec) spec))))
-                             (cond ((functionp fun-or-ctype)
-                                    (funcall fun-or-ctype (ensure-list spec)))
-                                   (fun-or-ctype)
-                                   ((or (and (consp spec) (symbolp (car spec))
-                                             (not (info :type :builtin (car spec))))
-                                        (and (symbolp spec) (not (info :type :builtin spec))))
-                                    (when (and *type-system-initialized*
-                                               (not (eq (info :type :kind spec)
-                                                        :forthcoming-defclass-type)))
-                                      (signal 'parse-unknown-type :specifier spec))
-                                    (setf cachep nil)
-                                    (make-unknown-type :specifier spec))
-                                   (t
-                                    (error "bad thing to be a type specifier: ~S"
-                                           spec))))))))))
-     (if cachep
-         result
-         ;; (The RETURN-FROM here inhibits caching; this does not only
-         ;; make sense from a compiler diagnostics point of view but
-         ;; is also indispensable for proper workingness of
-         ;; VALID-TYPE-SPECIFIER-P.)
-         (return-from values-specifier-type
-           result))))
+;;; As I understand things, :FORTHCOMING-DEFCLASS-TYPE behaves contrarily
+;;; to the CLHS intent, which is to make the type known to the compiler.
+;;; If we compile in one file:
+;;;  (DEFCLASS FRUITBAT () ())
+;;;  (DEFUN FRUITBATP (X) (TYPEP X 'FRUITBAT))
+;;; we see that it emits a call to %TYPEP with the symbol FRUITBAT as its
+;;; argument, whereas it should involve CLASSOID-CELL-TYPEP and LAYOUT-OF,
+;;; which (correctly) signals an error if the class were not defined by the
+;;; time of the call. Delayed re-parsing of FRUITBAT into any random specifier
+;;; at call time is wrong.
+;;;
+;;; FIXME: symbols which are :PRIMITIVE are inconsistently accepted as singleton
+;;; lists. e.g. (BIT) and (ATOM) are considered legal, but (FIXNUM) and
+;;; (CHARACTER) are not. It has to do with whether the primitive is actually
+;;; a DEFTYPE. The CLHS glossary implies that the singleton is *always* legal.
+;;;  "For every atomic type specifier, x, there is an _equivalent_ [my emphasis]
+;;;   compound type specifier with no arguments supplied, (x)."
+;;; By that same reasonining, is (x) accepted if x names a class?
+;;;
+
+;;; The xc host uses an ordinary hash table for memoization.
 #+sb-xc-host
 (let ((table (make-hash-table :test 'equal)))
-  (defun values-specifier-type (specifier)
+  (defun values-specifier-type-memo-wrapper (thunk specifier)
     (multiple-value-bind (type yesp) (gethash specifier table)
       (if yesp
           type
-          (setf (gethash specifier table)
-                (!values-specifier-type-body specifier)))))
+          (setf (gethash specifier table) (funcall thunk)))))
   (defun values-specifier-type-cache-clear ()
     (clrhash table)))
+;;; This cache is sized extremely generously, which has payoff
+;;; elsewhere: it improves the TYPE= and CSUBTYPEP functions,
+;;; since EQ types are an immediate win.
 #-sb-xc-host
-(defun-cached (values-specifier-type
-               :hash-function #'sxhash :hash-bits 10)
-    ((orig equal-but-no-car-recursion))
-  (!values-specifier-type-body orig))
+(sb!impl::!define-hash-cache values-specifier-type
+  ((orig equal-but-no-car-recursion)) ()
+  :hash-function #'sxhash :hash-bits 10)
+
+;;; The recursive ("-R" suffixed) entry point for this function
+;;; should be used for each nested parser invocation.
+(defun values-specifier-type-r (context type-specifier)
+  (declare (type cons context))
+  (labels ((fail (spec) ; Q: Shouldn't this signal a TYPE-ERROR ?
+             (error "bad thing to be a type specifier: ~S" spec))
+           (instance-to-ctype (x)
+             (flet ((translate (classoid)
+                      ;; Hmm, perhaps this should signal PARSE-UNKNOWN-TYPE
+                      ;; if CLASSOID is an instance of UNDEFINED-CLASSOID ?
+                      ;; Can that happen?
+                      (or (and (built-in-classoid-p classoid)
+                               (built-in-classoid-translation classoid))
+                          classoid)))
+               (cond ((classoid-p x) (translate x))
+                     ;; Avoid TYPEP on SB!MOP:EQL-SPECIALIZER and CLASS because
+                     ;; the fake metaobjects do not allow type analysis, and
+                     ;; would cause a compiler error as it tries to decide
+                     ;; whether any clause of this COND subsumes another.
+                     ;; Moreover, we don't require the host to support MOP.
+                     ((sb!pcl::classp x) (translate (sb!pcl::class-classoid x)))
+                     #-sb-xc-host
+                     ((sb!pcl::eql-specializer-p type-specifier)
+                      (make-eql-type
+                       (sb!mop:eql-specializer-object type-specifier)))
+                     (t (fail x))))))
+    (when (typep type-specifier 'instance)
+      (return-from values-specifier-type-r (instance-to-ctype type-specifier)))
+    (values-specifier-type-memo-wrapper
+     (lambda ()
+       (labels
+         ((recurse (spec)
+            (prog* ((head (if (listp spec) (car spec) spec))
+                    (builtin (if (symbolp head)
+                                 (info :type :builtin head)
+                                 (return (fail spec)))))
+              (when (deprecated-thing-p 'type head)
+                (setf (cdr context) nil)
+                (signal 'parse-deprecated-type :specifier spec))
+              (when (atom spec)
+                ;; If spec is non-atomic, the :BUILTIN value is inapplicable.
+                ;; There used to be compound builtins, but not any more.
+                (when builtin (return builtin))
+                (case (info :type :kind spec)
+                  (:instance (return (find-classoid spec)))
+                  (:forthcoming-defclass-type (go unknown))))
+              (awhen (info :type :translator head)
+                (return (or (funcall it context spec) (fail spec))))
+              ;; Expansion brings up an interesting question - should the cache
+              ;; contain entries for intermediary types? Say A -> B -> REAL.
+              ;; As it stands, we cache the ctype corresponding to A but not B.
+              (awhen (info :type :expander head)
+                (let ((expansion (funcall it (ensure-list spec))))
+                  (return (if (typep expansion 'instance)
+                              (instance-to-ctype expansion)
+                              (recurse expansion)))))
+              ;; If the spec is (X ...) and X has neither a translator
+              ;; nor expander, and is a builtin, such as FIXNUM, fail now.
+              ;; But - see FIXME at top - it would be consistent with
+              ;; DEFTYPE to reject spec only if not a singleton.
+              (when builtin (return (fail spec)))
+              ;; SPEC has a legal form, so return an unknown type.
+              (signal 'parse-unknown-type :specifier spec)
+             UNKNOWN
+              (setf (cdr context) nil)
+              (return (make-unknown-type :specifier spec)))))
+        (let ((result (recurse (uncross type-specifier))))
+          (if (cdr context) ; cacheable
+              result
+              ;; (The RETURN-FROM here inhibits caching; this makes sense
+              ;; not only from a compiler diagnostics point of view,
+              ;; but also for proper workingness of VALID-TYPE-SPECIFIER-P.
+              (return-from values-specifier-type-r result)))))
+     type-specifier)))
+(defun values-specifier-type (type-specifier)
+  (dx-let ((context (cons type-specifier t)))
+    (values-specifier-type-r context type-specifier)))
 
 ;;; This is like VALUES-SPECIFIER-TYPE, except that we guarantee to
 ;;; never return a VALUES type.
-(defun specifier-type (type-specifier)
-  (let ((ctype (values-specifier-type type-specifier)))
+(defun specifier-type-r (context type-specifier)
+  (let ((ctype (values-specifier-type-r context type-specifier)))
     (when (or (values-type-p ctype)
               ;; bootstrap magic :-(
               (and (named-type-p ctype)
                    (eq (named-type-name ctype) '*)))
       (error "VALUES type illegal in this context:~%  ~S" type-specifier))
     ctype))
+(defun specifier-type (type-specifier)
+  (dx-let ((context (cons type-specifier t)))
+    (specifier-type-r context type-specifier)))
 
+(defun single-value-specifier-type-r (context x)
+  (if (eq x '*) *universal-type* (specifier-type-r context x)))
 (defun single-value-specifier-type (x)
   (if (eq x '*)
       *universal-type*
@@ -981,7 +1038,7 @@
 Returns two values: the expansion, and a boolean that is true when
 expansion happened."
   (declare (type type-specifier type-specifier))
-  (declare (ignore env))
+  (declare (type lexenv-designator env) (ignore env))
   (let* ((spec type-specifier)
          (atom (if (listp spec) (car spec) spec))
          (expander (and (symbolp atom) (info :type :expander atom))))
@@ -1001,12 +1058,13 @@ expansion happened."
   "Takes and expands a type specifier repeatedly like MACROEXPAND.
 Returns two values: the expansion, and a boolean that is true when
 expansion happened."
-  (declare (type type-specifier type-specifier))
-  (multiple-value-bind (expansion flag)
+  ;; TYPE-SPECIFIER is of type TYPE-SPECIFIER, but it is preferable to
+  ;; defer to TYPEXPAND-1 for the typecheck. Similarly for ENV.
+  (multiple-value-bind (expansion expanded)
       (typexpand-1 type-specifier env)
-    (if flag
+    (if expanded
         (values (typexpand expansion env) t)
-        (values expansion flag))))
+        (values expansion expanded))))
 
 ;;; Note that the type NAME has been (re)defined, updating the
 ;;; undefined warnings and VALUES-SPECIFIER-TYPE cache.
