@@ -72,6 +72,9 @@
 (defun sb!c::control-stack-pointer-sap ()
   (sb!c::control-stack-pointer-sap))
 
+;;; FDEFN -> FUNCTION
+(defun sb!c:safe-fdefn-fun (x) (sb!c:safe-fdefn-fun x))
+
 ;;; Return the header typecode for FUNCTION. Can be set with SETF.
 (defun fun-subtype (function)
   (fun-subtype function))
@@ -211,21 +214,6 @@
 (defun code-instructions (code-obj)
   (code-instructions code-obj))
 
-(defun code-n-unboxed-data-words (code-obj)
-  ;; If the number of boxed words (from the header) is not the same as
-  ;; the displacement backwards from the first simple-fun to the header,
-  ;; then there are unboxed constants between the end of the boxed constants
-  ;; and the first simple-fun.
-  (let ((f (%code-entry-points code-obj)))
-    (or (and f
-             (let ((from (get-header-data code-obj))
-                   (to (ash (with-pinned-objects (f)
-                              (sap-ref-word (int-sap (get-lisp-obj-address f))
-                                            (- sb!vm:fun-pointer-lowtag)))
-                            (- sb!vm:n-widetag-bits))))
-               (and (< from to) (- to from))))
-        0)))
-
 ;;; Extract the INDEXth element from the header of CODE-OBJ. Can be
 ;;; set with SETF.
 (defun code-header-ref (code-obj index)
