@@ -179,7 +179,8 @@ invoked. In that case it will store into PLACE and start over."
   ;; DEBUG-NAME is called primarily for its side-effect of asserting
   ;; that (COMPILER-MACRO-FUNCTION x) is not a legal function name.
   (let ((def (make-macro-lambda (sb!c::debug-name 'compiler-macro name)
-                                lambda-list body 'define-compiler-macro name)))
+                                lambda-list body 'define-compiler-macro name
+                                :accessor 'sb!c::compiler-macro-args)))
     `(progn
           (eval-when (:compile-toplevel)
            (sb!c::%compiler-defmacro :compiler-macro-function ',name t))
@@ -244,9 +245,10 @@ invoked. In that case it will store into PLACE and start over."
                      for existing = (gethash k keys-seen)
                      do (when existing
                           (let ((sb!c::*current-path*
-                                 (when (boundp 'sb!c::*source-paths*)
-                                   (or (sb!c::get-source-path case)
-                                       sb!c::*current-path*))))
+                                  (when (boundp 'sb!c::*source-paths*)
+                                    (or (sb!c::get-source-path case)
+                                        (and (boundp 'sb!c::*current-path*)
+                                             sb!c::*current-path*)))))
                             (warn 'duplicate-case-key-warning
                                   :key k
                                   :case-kind name
