@@ -420,6 +420,9 @@
            (when present-p
              (warn "defining setf macro for ~S when ~S was previously ~
              treated as a function" name setf-fn-name)))
+          ;; This is a useless and unavoidable warning during self-build.
+          ;; cf. similar disabling of warning in WARN-IF-SETF-MACRO.
+          #-sb-xc-host
           (:defined
            ;; Somebody defined (SETF F) but then also said F has a macro.
            ;; A soft warning seems appropriate because in this case it's
@@ -639,6 +642,8 @@
     ;; always reference default's temp var to "use" it
     `(%puthash ,key ,hashtable ,(if constp newval `(progn ,default ,newval)))))
 
+(sb!xc:defsetf slot-value sb!pcl::set-slot-value)
+
 ;;; CMU CL had a comment here that:
 ;;;   Evil hack invented by the gnomes of Vassar Street (though not as evil as
 ;;;   it used to be.)  The function arg must be constant, and is converted to
