@@ -235,61 +235,43 @@
 (defun ansi-stream-read-line-from-frc-buffer (stream eof-error-p eof-value)
   (prepare-for-fast-read-char stream
     (declare (ignore %frc-method%))
+    (declare (type ansi-stream-cin-buffer %frc-buffer%))
     (let ((chunks-total-length 0)
           (chunks nil))
       (declare (type index chunks-total-length)
                (list chunks))
       (labels ((refill-buffer ()
-                 (prog1
-                     (fast-read-char-refill stream nil nil)
+                 (prog1 (fast-read-char-refill stream nil)
                    (setf %frc-index% (ansi-stream-in-index %frc-stream%))))
-               (newline-position ()
-                 (position #\Newline (the (simple-array character (*))
-                                       %frc-buffer%)
-                           :test #'char=
-                           :start %frc-index%))
-               (make-and-return-result-string (pos)
-                 (let* ((len (+ (- (or pos %frc-index%)
-                                   %frc-index%)
-                                chunks-total-length))
-                        (res (make-string len))
-                        (start 0))
-                   (declare (type index start))
-                   (when chunks
-                     (dolist (chunk (nreverse chunks))
-                       (declare (type (simple-array character) chunk))
-                       (replace res chunk :start1 start)
-                       (incf start (length chunk))))
-                   (unless (null pos)
+               (build-result (pos n-more-chars)
+                 (let ((res (make-string (+ chunks-total-length n-more-chars)))
+                       (start1 chunks-total-length))
+                   (declare (type index start1))
+                   (when (>= pos 0)
                      (replace res %frc-buffer%
-                              :start1 start
-                              :start2 %frc-index% :end2 pos)
+                              :start1 start1 :start2 %frc-index% :end2 pos)
                      (setf %frc-index% (1+ pos)))
                    (done-with-fast-read-char)
-                   (return-from ansi-stream-read-line-from-frc-buffer (values res (null pos)))))
-               (add-chunk ()
-                 (let* ((end (length %frc-buffer%))
-                        (len (- end %frc-index%))
-                        (chunk (make-string len)))
-                   (replace chunk %frc-buffer% :start2 %frc-index% :end2 end)
-                   (push chunk chunks)
-                   (incf chunks-total-length len)
-                   (when (refill-buffer)
-                     (make-and-return-result-string nil)))))
-        (declare (inline make-and-return-result-string
-                         refill-buffer))
-        (when (and (= %frc-index% +ansi-stream-in-buffer-length+)
-                   (refill-buffer))
+                   (dolist (chunk chunks res)
+                     (declare (type (simple-array character (*)) chunk))
+                     (decf start1 (length chunk))
+                     (replace res chunk :start1 start1)))))
+        (declare (inline refill-buffer))
+        (if (or (< %frc-index% +ansi-stream-in-buffer-length+) (refill-buffer))
+            (loop
+             (let ((pos (position #\Newline %frc-buffer%
+                                  :test #'char= :start %frc-index%)))
+               (when pos
+                 (return (values (build-result pos (- pos %frc-index%)) nil)))
+               (let ((chunk (subseq %frc-buffer% %frc-index%)))
+                 (incf chunks-total-length (length chunk))
+                 (push chunk chunks))
+               (unless (refill-buffer)
+                 (return (values (build-result -1 0) t)))))
           ;; EOF had been reached before we read anything
           ;; at all. Return the EOF value or signal the error.
-          (done-with-fast-read-char)
-          (return-from ansi-stream-read-line-from-frc-buffer
-            (values (eof-or-lose stream eof-error-p eof-value) t)))
-        (loop
-           (let ((pos (newline-position)))
-             (if pos
-                 (make-and-return-result-string pos)
-                 (add-chunk))))))))
+            (progn (done-with-fast-read-char)
+                   (eof-or-lose stream eof-error-p (values eof-value t))))))))
 
 #!-sb-fluid (declaim (inline ansi-stream-read-line))
 (defun ansi-stream-read-line (stream eof-error-p eof-value recursive-p)
@@ -372,7 +354,11 @@
     (when (minusp index) (error "nothing to unread"))
     (cond (buffer
            (setf (aref buffer index) character)
-           (setf (ansi-stream-in-index stream) index))
+           (setf (ansi-stream-in-index stream) index)
+           ;; Ugh. an ANSI-STREAM with a char buffer never gives a chance to
+           ;; the stream's misc routine to handle the UNREAD operation.
+           (when (ansi-stream-input-char-pos stream)
+             (decf (ansi-stream-input-char-pos stream))))
           (t
            (funcall (ansi-stream-misc stream) stream
                     :unread character)))))
@@ -528,7 +514,13 @@
 ;;; the IN-BUFFER for text streams. There is definitely an IN-BUFFER,
 ;;; and hence must be an N-BIN method. It's also called by other stream
 ;;; functions which directly peek into the frc buffer.
-(defun fast-read-char-refill (stream eof-error-p eof-value)
+;;; If EOF is hit and EOF-ERROR-P is false, then return NIL,
+;;; otherwise return the new index into CIN-BUFFER.
+(defun fast-read-char-refill (stream eof-error-p)
+  (when (ansi-stream-input-char-pos stream)
+    ;; Characters between (ANSI-STREAM-IN-INDEX %FRC-STREAM%)
+    ;; and +ANSI-STREAM-IN-BUFFER-LENGTH+ have to be re-scanned.
+    (update-input-char-pos stream))
   (let* ((ibuf (ansi-stream-cin-buffer stream))
          (count (funcall (ansi-stream-n-bin stream)
                          stream
@@ -557,15 +549,22 @@
            (let* ((index (1- +ansi-stream-in-buffer-length+))
                   (value (funcall (ansi-stream-in stream) stream nil :eof)))
              (cond
+               ;; When not signaling an error, it is important that IN-INDEX
+               ;; be set to +ANSI-STREAM-IN-BUFFER-LENGTH+ here, even though
+               ;; DONE-WITH-FAST-READ-CHAR will do the same, thereby writing
+               ;; the caller's %FRC-INDEX% (= +ANSI-STREAM-IN-BUFFER-LENGTH+)
+               ;; into the slot. But because we've already bumped INPUT-CHAR-POS
+               ;; and scanned characters between the original %FRC-INDEX%
+               ;; and the buffer end (above), we must *not* do that again.
                ((eql value :eof)
                 ;; definitely EOF now
                 (setf (ansi-stream-in-index stream)
                       +ansi-stream-in-buffer-length+)
-                (values t (eof-or-lose stream eof-error-p eof-value)))
+                (eof-or-lose stream eof-error-p nil))
                ;; we resynced or were given something instead
                (t
                 (setf (aref ibuf index) value)
-                (values nil (setf (ansi-stream-in-index stream) index))))))
+                (setf (ansi-stream-in-index stream) index)))))
           (t
            (when (/= start +ansi-stream-in-buffer-extra+)
              (#.(let* ((n-character-array-bits
@@ -579,8 +578,7 @@
                 ibuf +ansi-stream-in-buffer-extra+
                 ibuf start
                 count))
-           (values nil
-                   (setf (ansi-stream-in-index stream) start))))))
+           (setf (ansi-stream-in-index stream) start)))))
 
 ;;; This is similar to FAST-READ-CHAR-REFILL, but we don't have to
 ;;; leave room for unreading.
@@ -1296,8 +1294,7 @@
                       (out #'string-ouch)
                       (sout #'string-sout)
                       (misc #'string-out-misc))
-            (:constructor make-string-output-stream
-                          (&key (element-type 'character)))
+            (:constructor %make-string-output-stream (element-type))
             (:copier nil)
             (:predicate nil))
   ;; The string we throw stuff in.
@@ -1316,15 +1313,19 @@
   ;; end of the stream.
   (index-cache 0 :type index)
   ;; Requested element type
+  ;; FIXME: there seems to be no way to skip the type-check in the ctor,
+  ;; which is redundant with the check in MAKE-STRING-OUTPUT-STREAM.
   (element-type 'character :type type-specifier
                            :read-only t))
 
 (declaim (freeze-type string-output-stream))
-
-#!+sb-doc
-(setf (fdocumentation 'make-string-output-stream 'function)
+(defun make-string-output-stream (&key (element-type 'character))
+  #!+sb-doc
   "Return an output stream which will accumulate all output given it for the
-benefit of the function GET-OUTPUT-STREAM-STRING.")
+benefit of the function GET-OUTPUT-STREAM-STRING."
+  (if (csubtypep (specifier-type element-type) (specifier-type 'character))
+      (%make-string-output-stream element-type)
+      (error "~S is not a subtype of CHARACTER" element-type)))
 
 ;;; Pushes the current segment onto the prev-list, and either pops
 ;;; or allocates a new one.
@@ -2049,8 +2050,7 @@ benefit of the function GET-OUTPUT-STREAM-STRING.")
       (unless %frc-buffer%
         (return-from ansi-stream-read-string-from-frc-buffer nil))
       (labels ((refill-buffer ()
-                 (prog1
-                     (fast-read-char-refill stream nil nil)
+                 (prog1 (fast-read-char-refill stream nil)
                    (setf %frc-index% (ansi-stream-in-index %frc-stream%))))
                (add-chunk ()
                  (let* ((end (length %frc-buffer%))
@@ -2067,16 +2067,15 @@ benefit of the function GET-OUTPUT-STREAM-STRING.")
                               :end2 (+ %frc-index% len)))
                    (incf read len)
                    (incf %frc-index% len)
-                   (when (or (eql needed read)
-                             (refill-buffer))
+                   (when (or (eql needed read) (not (refill-buffer)))
                      (done-with-fast-read-char)
                      (return-from ansi-stream-read-string-from-frc-buffer
                        (+ start read))))))
         (declare (inline refill-buffer))
         (when (and (= %frc-index% +ansi-stream-in-buffer-length+)
-                   (refill-buffer))
+                   (not (refill-buffer)))
           ;; EOF had been reached before we read anything
-          ;; at all. Return the EOF value or signal the error.
+          ;; at all. But READ-SEQUENCE never signals an EOF error.
           (done-with-fast-read-char)
           (return-from ansi-stream-read-string-from-frc-buffer start))
         (loop (add-chunk))))))
