@@ -105,10 +105,10 @@
                 ;; order dependencies from the old PCL defsys.lisp
                 ;; dependency database.
                 #+nil "src/pcl/walk" ; #+NIL = moved to build-order.lisp-expr
-                "SRC;PCL;EARLY-LOW"
+                #+nil "SRC;PCL;EARLY-LOW"
                 "SRC;PCL;MACROS"
                 "SRC;PCL;COMPILER-SUPPORT"
-                "SRC;PCL;LOW"
+                #+nil "SRC;PCL;LOW"
                 #+nil "SRC;PCL;SLOT-NAME" ; moved to build-order.lisp-expr
                 "SRC;PCL;DEFCLASS"
                 "SRC;PCL;DEFS"
@@ -229,6 +229,10 @@
       'sb-kernel:classoid (lambda (stream obj)
                             (print-unreadable-object (obj stream :type t)
                               (write (sb-kernel:classoid-name obj) :stream stream))))
+     (set-pprint-dispatch
+      'sb-kernel:ctype (lambda (stream obj)
+                         (print-unreadable-object (obj stream :type t)
+                           (prin1 (sb-kernel:type-specifier obj) stream))))
      (set-pprint-dispatch
       'package (lambda (stream obj)
                  (print-unreadable-object (obj stream :type t)
@@ -245,6 +249,19 @@
       'restart (lambda (stream obj)
                  (print-unreadable-object (obj stream :type t :identity t)
                    (write (restart-name obj) :stream stream))))
+     ;; These next two are coded in a totally brittle way, but no more wrong
+     ;; than typing the decoding expressions into the debugger to decipher
+     ;; a backtrace. Anyway, if it ceases to print right, just fix it again!
+     (set-pprint-dispatch
+      'generic-function
+      (lambda (stream obj)
+        (format stream "<~S ~S>" (type-of obj)
+                (svref (sb-kernel:%funcallable-instance-info obj 1) 5))))
+     (set-pprint-dispatch
+      'class
+      (lambda (stream obj)
+        (format stream "<~S ~S>" (type-of obj)
+                (svref (sb-kernel:%instance-ref obj 1) 3))))
      (with-compilation-unit ()
        (let ((*compile-print* nil))
          (do-srcs pcl-srcs)))
