@@ -968,11 +968,11 @@
                   (when step-instrumenting
                     (assemble ()
                       ;; Get the symbol-value of SB!IMPL::*STEPPING*
-                      ;; KLUDGE: ... into LIP.  Either it's NIL or it
+                      ;; KLUDGE: ... into LIP.  Either it's zero or it
                       ;; isn't, and even taking a stray interrupt and
                       ;; GC can't screw that up.
                       (load-symbol-value lip sb!impl::*stepping*)
-                      (inst cmp lip null-tn)
+                      (inst cmp lip 0)
                       ;; If it's not null, trap.
                       (inst b :eq step-done-label)
                       ;; CONTEXT-PC will be pointing here when the
@@ -1212,8 +1212,8 @@
   (:vop-var vop)
   (:generator 3
     (load-symbol-value stepping sb!impl::*stepping*)
-    ;; If it's not NIL, trap.
-    (inst cmp stepping null-tn)
+    ;; If it's not zero, trap.
+    (inst cmp stepping 0)
     (inst b :eq DONE)
     ;; CONTEXT-PC will be pointing here when the interrupt is handled,
     ;; not after the BREAK.
