@@ -84,7 +84,14 @@
                (accept (lambda-list-keyword-mask
                         '(&optional &rest &more &key &allow-other-keys &aux)))
                (condition-class 'simple-program-error)
-               silent
+               ;; For internal method functions, just shut up about everything
+               ;; that we could style-warn about. Interpreters tend to scan the
+               ;; lambda list at various inopportune times depending on strategy
+               ;; (macro caching, etc) and it's really annoying to see the
+               ;; &OPTIONAL/&KEY message randomly repeated, especially if it's
+               ;; someone else's code. Fwiw, 'full-eval' muffles warnings during
+               ;; all calls to this parser anyway.
+               (silent (typep list '(cons (eql sb!pcl::.pv.))))
           &aux (seen 0) required optional rest more keys aux env whole tail
                (rest-bits 0))
   (declare (optimize speed))
@@ -111,6 +118,14 @@
                       (and (typep name 'simple-base-string)
                            (plusp (length name))
                            (char= (char name 0) #\&)))))
+             (check-suspicious (kind form)
+               (and (probably-ll-keyword-p form)
+                    (member form sb!xc:lambda-list-keywords)
+                    (report-suspicious kind form)))
+             (report-suspicious (kind what)
+               (style-warn "suspicious ~A ~S in lambda list: ~S."
+                           kind what list)
+               nil) ; Avoid "return convention is not fixed" optimizer note
              (need-arg (state)
                (croak "expecting variable after ~A in: ~S" state list))
              (need-symbol (x why)
@@ -143,7 +158,9 @@
                  ;; Expecting a callee to understand how to signal conditions
                  ;; tailored to a particular caller is not how things are
                  ;; supposed to work.
-                 (funcall (if (destructuring-p) 'error 'compiler-error)
+                 (funcall (if (or (destructuring-p) (eq context 'defmethod))
+                              'error
+                              'compiler-error)
                           condition-class
                           :format-control string :format-arguments l))))
       (prog ((input list)
@@ -202,6 +219,7 @@
                           (:values-type "a VALUES type specifier")
                           (:macro "a macro lambda list")
                           (destructuring-bind "a destructuring lambda list")
+                          (defmethod "a specialized lambda list")
                           (t context))))
                    (croak "~A is not allowed in ~A: ~S" arg where list)))
 
@@ -234,9 +252,7 @@
                              arg list)))
                (go LOOP)))
            ;; Fell through, so warn if desired, and fall through some more.
-           (unless silent
-             (style-warn
-              "suspicious variable ~S in lambda list: ~S." arg list)))
+           (unless silent (report-suspicious "variable" arg)))
 
          ;; Handle a lambda variable
          (when (logbitp state (bits &allow-other-keys ; Not a collecting state.
@@ -275,7 +291,12 @@
                        #'need-symbol #'need-bindable)
                    (car whole) "&WHOLE argument"))
         (dolist (arg required)
-          (need-bindable arg "Required argument"))
+          (if (eq context 'defmethod)
+              (unless (or (and (symbolp arg) (not (null arg)))
+                          (and (listp arg) (singleton-p (cdr arg))))
+                (croak "arg is not a non-NIL symbol or a list of two elements: ~A"
+                       arg))
+              (need-bindable arg "Required argument")))
         ;; FIXME: why not check symbol-ness of supplied-p variables now?
         (flet ((scan-opt/key (list what-kind description)
                  (dolist (arg list)
@@ -286,7 +307,8 @@
                      (destructuring-bind (var &optional default sup-p) arg
                        (if (and (consp var) (eq what-kind '&key))
                            (destructuring-bind (keyword-name var) var
-                             (declare (ignore keyword-name))
+                             (unless (symbolp keyword-name)
+                               (croak "keyword-name in ~S is not a symbol" arg))
                              (need-bindable var description))
                            (need-bindable var description))
                        ;; Inform the user about a possibly malformed
@@ -297,15 +319,8 @@
                        ;; and an illegal name for a DEFVAR or such,
                        ;; being in the CL package.
                        (unless silent
-                         (when (and (probably-ll-keyword-p default)
-                                    (member default sb!xc:lambda-list-keywords))
-                           (style-warn "suspicious default ~S in lambda list: ~S."
-                                       default list))
-                         (when (and (probably-ll-keyword-p sup-p)
-                                    (member sup-p sb!xc:lambda-list-keywords))
-                           (style-warn
-                            "suspicious supplied-p variable ~S in lambda list: ~S."
-                            sup-p list))))))))
+                         (check-suspicious "default" default)
+                         (check-suspicious "supplied-p variable" sup-p)))))))
           (scan-opt/key optional '&optional "&OPTIONAL parameter name")
           (when rest
             (need-bindable (car rest) "&REST argument"))
@@ -1045,8 +1060,8 @@
 ;;; The CLtl2 name for this operation is PARSE-MACRO.
 (defun make-macro-lambda
     (lambda-name lambda-list body kind name
-     &key ((:environment envp) t) (doc-string-allowed :internal)
-           (wrap-block name))
+     &key (accessor 'cdr) (doc-string-allowed :internal)
+          ((:environment envp) t) (wrap-block name))
   (declare (type (member t nil :ignore) envp))
   (declare (type (member nil :external :internal) doc-string-allowed))
   (binding* (((forms decls docstring)
@@ -1076,9 +1091,7 @@
                       (when whole `((,(car whole) ,ll-whole)))))
              ;; Drop &WHOLE and &ENVIRONMENT
              (new-ll (make-lambda-list llks nil req opt rest keys aux))
-             (parse (parse-ds-lambda-list new-ll))
-             (arg-accessor
-              (if (eq kind 'define-compiler-macro) 'compiler-macro-args 'cdr)))
+             (parse (parse-ds-lambda-list new-ll)))
     ;; Signal a style warning for duplicate names, but disregard &AUX variables
     ;; because most folks agree that (LET* ((X (F)) (X (G X))) ..) makes sense
     ;; - some would even say that it is idiomatic - and &AUX bindings are just
@@ -1104,7 +1117,7 @@
                                            `(:special-form . ,name)
                                            `(:macro ,name . ,kind)))
                       '(destructuring-bind))
-                   ,new-ll (,arg-accessor ,ll-whole)
+                   ,new-ll (,accessor ,ll-whole)
                  ,@decls
                  ,@(if wrap-block
                        `((block ,(fun-name-block-name name) ,@forms))
