@@ -158,7 +158,7 @@
                                       result-type)))
                    (if (array-type-p result-ctype)
                        (let ((dims (array-type-dimensions result-ctype)))
-                         (unless (and (listp dims) (= (length dims) 1))
+                         (unless (singleton-p dims)
                            (give-up-ir1-transform "invalid sequence type"))
                          (let ((dim (first dims)))
                            (if (eq dim '*)
@@ -295,20 +295,20 @@
                      (list (values `(push funcall-result acc)
                                    `(nreverse acc))))
                  (catch-give-up-ir1-transform
-                     `(lambda (result-type fun ,@seq-args)
-                        (declare (ignore result-type))
-                        (let ((fun (%coerce-callable-to-fun fun))
-                              (acc nil))
-                          (declare (type list acc))
-                          (declare (ignorable acc))
-                          ,(build-sequence-iterator
-                            all-seqs seq-args
-                            :result result
-                            :body push-dacc
-                            :fast (policy node (> speed space)))))
+                     (`(lambda (result-type fun ,@seq-args)
+                         (declare (ignore result-type))
+                         (let ((fun (%coerce-callable-to-fun fun))
+                               (acc nil))
+                           (declare (type list acc))
+                           (declare (ignorable acc))
+                           ,(build-sequence-iterator
+                             all-seqs seq-args
+                             :result result
+                             :body push-dacc
+                             :fast (policy node (> speed space))))))
                    (if (and (null result-type-value) (null seqs))
                        '(%map-for-effect-arity-1 fun seq)
-                       (give-up))))))))))
+                       (%give-up))))))))))
 
 ;;; MAP-INTO
 (deftransform map-into ((result fun &rest seqs)
@@ -320,33 +320,33 @@
          (non-complex-vector-type-p (csubtypep result-type
                                                (specifier-type '(simple-array * 1)))))
     (catch-give-up-ir1-transform
-        `(lambda (result fun ,@seqs-names)
-           ,(if (and (policy node (> speed space))
-                     (not non-complex-vector-type-p))
-                (let ((data  (gensym "DATA"))
-                      (start (gensym "START"))
-                      (end   (gensym "END")))
-                  `(with-array-data ((,data result)
-                                     (,start)
-                                     (,end))
-                     (declare (ignore ,end))
-                     ,(build-sequence-iterator
-                       seqs seqs-names
-                       :result '(when (array-has-fill-pointer-p result)
-                                 (setf (fill-pointer result) index))
-                       :into 'result
-                       :body `(locally (declare (optimize (insert-array-bounds-checks 0)))
-                                (setf (aref ,data (truly-the index (+ index ,start)))
-                                      funcall-result))
-                       :fast t)))
-                (build-sequence-iterator
-                 seqs seqs-names
-                 :result '(when (array-has-fill-pointer-p result)
-                           (setf (fill-pointer result) index))
-                 :into 'result
-                 :body '(locally (declare (optimize (insert-array-bounds-checks 0)))
-                         (setf (aref result index) funcall-result))))
-           result)
+        (`(lambda (result fun ,@seqs-names)
+            ,(if (and (policy node (> speed space))
+                      (not non-complex-vector-type-p))
+                 (let ((data  (gensym "DATA"))
+                       (start (gensym "START"))
+                       (end   (gensym "END")))
+                   `(with-array-data ((,data result)
+                                      (,start)
+                                      (,end))
+                      (declare (ignore ,end))
+                      ,(build-sequence-iterator
+                        seqs seqs-names
+                        :result '(when (array-has-fill-pointer-p result)
+                                  (setf (fill-pointer result) index))
+                        :into 'result
+                        :body `(locally (declare (optimize (insert-array-bounds-checks 0)))
+                                 (setf (aref ,data (truly-the index (+ index ,start)))
+                                       funcall-result))
+                        :fast t)))
+                 (build-sequence-iterator
+                  seqs seqs-names
+                  :result '(when (array-has-fill-pointer-p result)
+                            (setf (fill-pointer result) index))
+                  :into 'result
+                  :body '(locally (declare (optimize (insert-array-bounds-checks 0)))
+                          (setf (aref result index) funcall-result))))
+            result))
       (cond #-sb-xc-host
             ;; %%vector-map-into-funs%% is not defined in xc
             ;; if something needs to be faster in the compiler, it
@@ -363,7 +363,7 @@
                  `(progn (,mapper result 0 (length result) (%coerce-callable-to-fun fun) seqs)
                          result))))
             (t
-             (give-up))))))
+             (%give-up))))))
 
 
 ;;; FIXME: once the confusion over doing transforms with known-complex
@@ -631,9 +631,9 @@
                   (kind (cond ((sb!vm:saetp-fixnum-p saetp) :tagged)
                               ((member element-type '(character base-char)) :char)
                               ((eq element-type 'single-float) :single-float)
-                              #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+                              #!+64-bit
                               ((eq element-type 'double-float) :double-float)
-                              #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+                              #!+64-bit
                               ((equal element-type '(complex single-float))
                                :complex-single-float)
                               (t
@@ -658,11 +658,11 @@
                                          tmp)
                                         (:single-float
                                          (single-float-bits tmp))
-                                        #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+                                        #!+64-bit
                                         (:double-float
                                          (logior (ash (double-float-high-bits tmp) 32)
                                                  (double-float-low-bits tmp)))
-                                        #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+                                        #!+64-bit
                                         (:complex-single-float
                                          (logior (ash (single-float-bits (imagpart tmp)) 32)
                                                  (ldb (byte 32 0)
@@ -685,11 +685,11 @@
                                                     `item)
                                                    (:single-float
                                                     `(single-float-bits item))
-                                                   #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+                                                   #!+64-bit
                                                    (:double-float
                                                     `(logior (ash (double-float-high-bits item) 32)
                                                              (double-float-low-bits item)))
-                                                   #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
+                                                   #!+64-bit
                                                    (:complex-single-float
                                                     `(logior (ash (single-float-bits (imagpart item)) 32)
                                                              (ldb (byte 32 0)
@@ -1042,23 +1042,20 @@
                    (%vector-raw-bits src (1- i))))
            (values))))))
 
-#.(loop for i = 1 then (* i 2)
-        collect `(deftransform ,(intern (format nil "UB~D-BASH-COPY" i)
-                                        "SB!KERNEL")
-                                                        ((src src-offset
-                                                          dst dst-offset
-                                                          length)
-                                                        ((simple-unboxed-array (*))
-                                                         (constant-arg index)
-                                                         (simple-unboxed-array (*))
-                                                         (constant-arg index)
-                                                         index)
-                                                        *)
-                  (frob-bash-transform src src-offset
-                                       dst dst-offset length
-                                       ,(truncate sb!vm:n-word-bits i))) into forms
-        until (= i sb!vm:n-word-bits)
-        finally (return `(progn ,@forms)))
+#.
+(let ((arglist '((src src-offset dst dst-offset length)
+                 ((simple-unboxed-array (*)) (constant-arg index)
+                  (simple-unboxed-array (*)) (constant-arg index)
+                  index)
+                 *)))
+  (loop for i = 1 then (* i 2)
+     for name = (intern (format nil "UB~D-BASH-COPY" i) "SB!KERNEL")
+     collect `(deftransform ,name ,arglist
+                (frob-bash-transform src src-offset
+                                     dst dst-offset length
+                                     ,(truncate sb!vm:n-word-bits i))) into forms
+     until (= i sb!vm:n-word-bits)
+     finally (return `(progn ,@forms))))
 
 ;;; We expand copy loops inline in SUBSEQ and COPY-SEQ if we're copying
 ;;; arrays with elements of size >= the word size.  We do this because
