@@ -254,33 +254,11 @@
   (fun (missing-arg) :type function)
   (before-address nil :type (member t nil)))
 
-(defstruct (segment (:conc-name seg-)
-                    (:constructor %make-segment)
-                    (:copier nil))
-  (sap-maker (missing-arg)
-             :type (function () sb!sys:system-area-pointer))
-  ;; Length in bytes of the range of memory covered by this segment.
-  (length 0 :type disassem-length)
-  ;; Length of the memory range excluding any trailing untagged data.
-  ;; Defaults to 'length' but could be shorter.
-  (opcodes-length 0 :type disassem-length)
-  ;; The number of pad bytes after opcodes-length that should be dumped
-  ;; as raw bytes prior to displaying raw bytes that are genuinely data.
-  (opcode-trailer-length 0 :type disassem-length)
-  (virtual-location 0 :type address)
-  (storage-info nil :type (or null storage-info))
-  ;; For backends which support unboxed constants within the segment,
-  ;; they are collected into unboxed-refs so that they can be shown
-  ;; in their correct size according to the referencing instruction.
-  (unboxed-refs nil :type list) ; alist of (offset . size)
-  (code nil :type (or null sb!kernel:code-component))
-  (hooks nil :type list))
 (def!method print-object ((seg segment) stream)
   (print-unreadable-object (seg stream :type t)
     (let ((addr (sb!sys:sap-int (funcall (seg-sap-maker seg)))))
-      (format stream "#X~X[~W]~:[ (#X~X)~;~*~]~@[ in ~S~]"
-              addr
-              (seg-length seg)
+      (format stream "#X~X..~X[~W]~:[ (#X~X)~;~*~]~@[ in ~S~]"
+              addr (+ addr (seg-length seg)) (seg-length seg)
               (= (seg-virtual-location seg) addr)
               (seg-virtual-location seg)
               (seg-code seg)))))
@@ -518,6 +496,15 @@
            (type (or null stream) stream))
 
   (let ((ispace (get-inst-space))
+        (data-end-offset
+         ;; If the segment starts with unboxed data,
+         ;; dump some number of words using the .WORD pseudo-ops.
+         (if (and (seg-unboxed-data-range segment)
+                  (= (segment-offs-to-code-offs 0 segment)
+                     (car (seg-unboxed-data-range segment))))
+             (code-offs-to-segment-offs (cdr (seg-unboxed-data-range segment))
+                                        segment)
+             0)) ; sentinel value
         (prefix-p nil) ; just processed a prefix inst
         (prefix-len 0) ; sum of lengths of any prefix instruction(s)
         (prefix-print-names nil)) ; reverse list of prefixes seen
@@ -542,6 +529,17 @@
         (print-current-address stream dstate))
       (call-offs-hooks nil stream dstate)
 
+      (when (< (dstate-cur-offs dstate) data-end-offset)
+        (sb!sys:without-gcing
+         (format stream "~A  #x~v,'0x" '.word
+                 (* 2 sb!vm:n-word-bytes)
+                 (sap-ref-int (funcall (seg-sap-maker segment))
+                              (dstate-cur-offs dstate)
+                              sb!vm:n-word-bytes
+                              (dstate-byte-order dstate))))
+        (setf (dstate-next-offs dstate)
+              (+ (dstate-cur-offs dstate) sb!vm:n-word-bytes)))
+
       (unless (> (dstate-next-offs dstate) (dstate-cur-offs dstate))
         (sb!sys:without-gcing
          (setf (dstate-segment-sap dstate) (funcall (seg-sap-maker segment)))
@@ -905,6 +903,7 @@
 #!-sb-fluid (declaim (inline sap-maker))
 (defun sap-maker (function input offset)
   (declare (optimize (speed 3))
+           (muffle-conditions compiler-note)
            (type (function (t) sb!sys:system-area-pointer) function)
            (type offset offset))
   (let ((old-sap (sb!sys:sap+ (funcall function input) offset)))
@@ -932,6 +931,7 @@
 
 (defun memory-sap-maker (address)
   (declare (optimize (speed 3))
+           (muffle-conditions compiler-note)
            (type address address))
   (let ((sap (sb!sys:int-sap address)))
     (lambda () sap)))
@@ -961,7 +961,14 @@
            :virtual-location (or virtual-location
                                  (sb!sys:sap-int (funcall sap-maker)))
            :hooks hooks
-           :code code)))
+           :code code
+           :unboxed-data-range
+           (and code
+                (let ((n-words (sb!kernel:code-n-unboxed-data-words code))
+                      (start (sb!kernel:get-header-data code)))
+                  (and (plusp n-words)
+                       (cons (* sb!vm:n-word-bytes start)
+                             (* sb!vm:n-word-bytes (+ start n-words)))))))))
     (add-debugging-hooks segment debug-fun source-form-cache)
     (add-fun-header-hooks segment)
     segment))
@@ -1047,10 +1054,6 @@
 (defstruct (location-group (:copier nil))
   (locations #() :type (vector (or list fixnum))))
 
-(defstruct (storage-info (:copier nil))
-  (groups nil :type list)               ; alist of (name . location-group)
-  (debug-vars #() :type vector))
-
 ;;; Return the vector of DEBUG-VARs currently associated with DSTATE.
 (defun dstate-debug-vars (dstate)
   (declare (type disassem-state dstate))
@@ -1432,14 +1435,7 @@
            (funcall printer chunk inst stream dstate))))
      segment
      dstate
-     stream)
-    ;; "unboxed data" are more general than just large constants,
-    ;; but presently are comprised only of those. It would have made
-    ;; sense to featurize this on inline-constants, however it turned
-    ;; out to be difficult to get x86 (-32) to work, which as it happens
-    ;; is the only other backend that has the inline-constants feature.
-    #!+x86-64
-    (disassemble-unboxed-data segment stream dstate)))
+     stream)))
 
 ;;; Disassemble the machine code instructions in each memory segment
 ;;; in SEGMENTS in turn to STREAM.
@@ -1471,10 +1467,6 @@
                     (seg-virtual-location seg) i n-segments))
           (disassemble-segment seg stream dstate))))))
 
-#!-x86-64
-(defun determine-opcode-bounds (seglist dstate)
-  (declare (ignore seglist dstate)))
-
 
 ;;;; top level functions
 
@@ -1487,7 +1479,6 @@
            (type (member t nil) use-labels))
   (let* ((dstate (make-dstate))
          (segments (get-fun-segments fun)))
-    (determine-opcode-bounds segments dstate)
     (when use-labels
       (label-segments segments dstate))
     (disassemble-segments segments stream dstate)))
@@ -1501,6 +1492,7 @@
         ((typep thing 'sb!pcl::%method-function)
          ;; in a %METHOD-FUNCTION, the user code is in the fast function, so
          ;; we to disassemble both.
+         ;; FIXME: interpreted methods need to be compiled as above.
          (list thing (sb!pcl::%method-function-fast-function thing)))
         ((functionp thing)
          thing)
@@ -1535,10 +1527,7 @@
            (disassemble-fun fun
                             :stream stream
                             :use-labels use-labels)))
-    (let ((funs (compiled-funs-or-lose object)))
-      (if (listp funs)
-          (dolist (fun funs) (disassemble1 fun))
-          (disassemble1 funs))))
+    (mapc #'disassemble1 (ensure-list (compiled-funs-or-lose object))))
   nil)
 
 ;;; Disassembles the given area of memory starting at ADDRESS and
@@ -1592,7 +1581,6 @@
               code-component))
          (dstate (make-dstate))
          (segments (get-code-segments code-component)))
-    (determine-opcode-bounds segments dstate)
     (when use-labels
       (label-segments segments dstate))
     (disassemble-segments segments stream dstate)))
@@ -1606,16 +1594,12 @@
 
 ;;; Disassemble the machine code instructions associated with
 ;;; ASSEM-SEGMENT (of type assem:segment).
-;;; The logic to determine opcode bounds is the same as for the above cases,
-;;; which is unfortunately Rube-Goldberg-esque here, as the assembler knows the
-;;; bounds, but has already combined multiple segments. This could be improved.
 (defun disassemble-assem-segment (assem-segment stream)
   (declare (type sb!assem:segment assem-segment)
            (type stream stream))
   (let ((dstate (make-dstate))
         (disassem-segments
          (list (assem-segment-to-disassem-segment assem-segment))))
-    (determine-opcode-bounds disassem-segments dstate)
     (label-segments disassem-segments dstate)
     (disassemble-segments disassem-segments stream dstate)))
 
@@ -1670,7 +1654,6 @@
 ;;; constant area of the code-object in the current segment and T, or
 ;;; NIL and NIL if there is no code-object in the current segment.
 (defun get-code-constant (byte-offset dstate)
-  #!+sb-doc
   (declare (type offset byte-offset)
            (type disassem-state dstate))
   (let ((code (seg-code (dstate-segment dstate))))
@@ -1683,36 +1666,25 @@
          t)
         (values nil nil))))
 
-(defstruct code-constant-raw value)
-(def!method print-object ((self code-constant-raw) stream)
-  ;; A raw code constant is never a Lisp object, so if it is a cons,
-  ;; then it's the address of raw data- mnemonic is that (x) is like [x].
-  ;; Ideally we would want to show is as [Label] and then label the
-  ;; corresponding unboxed data.
-  (if (listp (code-constant-raw-value self))
-      (format stream "[#x~x]" (car (code-constant-raw-value self)))
-      (format stream "#x~8,'0x" (code-constant-raw-value self))))
-
 (defun get-code-constant-absolute (addr dstate &optional width)
   (declare (type address addr))
   (declare (type disassem-state dstate))
+  (declare (ignore width))
   (let ((code (seg-code (dstate-segment dstate))))
     (if (null code)
       (return-from get-code-constant-absolute (values nil nil)))
+    ;; This WITHOUT-GCING, while not technically broken, is extremely deceptive
+    ;; because if it is really needed, then this function has a broken API.
+    ;; Since ADDR comes in as absolute, CODE must not move between the caller's
+    ;; computation and the comparison below. But we're already in WITHOUT-GCING
+    ;; in MAP-SEGMENT-INSTRUCTIONS, so, who cares, I guess?
     (sb!sys:without-gcing
      (let* ((n-header-bytes (* (sb!kernel:get-header-data code) sb!vm:n-word-bytes))
-            (n-code-bytes (sb!kernel:%code-code-size code))
             (header-addr (- (sb!kernel:get-lisp-obj-address code)
                             sb!vm:other-pointer-lowtag))
             (code-start (+ header-addr n-header-bytes)))
          (cond ((< header-addr addr code-start)
                 (values (sb!sys:sap-ref-lispobj (sb!sys:int-sap addr) 0) t))
-               ;; guess it's a non-descriptor constant from the instructions
-               ((and (eq width :qword)
-                     (< code-start addr (+ code-start n-code-bytes)))
-                (values (make-code-constant-raw
-                         :value (sb!sys:sap-ref-64 (sb!sys:int-sap addr) 0))
-                        t))
                (t
                 (values nil nil)))))))
 
@@ -1759,6 +1731,7 @@
            (type (unsigned-byte 16) offset)
            (type (member 1 2 4 8) length)
            (type (member :little-endian :big-endian) byte-order)
+           (muffle-conditions compiler-note) ; integer coercion, oh well
            (optimize (speed 3) (safety 0)))
   (ecase length
     (1 (sb!sys:sap-ref-8 sap offset))
@@ -1957,7 +1930,7 @@
       (return (note (lambda (s) (prin1 symbol s)) dstate)))))
 
 (defun get-internal-error-name (errnum)
-  (cdr (svref sb!c:*backend-internal-errors* errnum)))
+  (cdr (svref sb!c:+backend-internal-errors+ errnum)))
 
 (defun get-sc-name (sc-offs)
   (sb!c:location-print-name
@@ -1997,18 +1970,26 @@
     (when stream
       (setf (dstate-cur-offs dstate)
             (dstate-next-offs dstate))
-      (flet ((emit-err-arg (note)
+      (flet ((emit-err-arg ()
                (let ((num (pop lengths)))
                  (print-notes-and-newline stream dstate)
                  (print-current-address stream dstate)
                  (print-inst num stream dstate)
                  (print-bytes num stream dstate)
-                 (incf (dstate-cur-offs dstate) num)
-                 (when note
-                   (note note dstate)))))
-        (emit-err-arg nil)
-        (emit-err-arg (symbol-name (get-internal-error-name errnum)))
+                 (incf (dstate-cur-offs dstate) num)))
+             (emit-note (note)
+               (when note
+                 (note note dstate))))
+        (emit-err-arg)
+        (emit-err-arg)
+        (emit-note (symbol-name (get-internal-error-name errnum)))
         (dolist (sc-offs sc-offsets)
-          (emit-err-arg (get-sc-name sc-offs)))))
+          (emit-err-arg)
+          (if (= (sb!c:sc-offset-scn sc-offs)
+                 sb!vm:constant-sc-number)
+              (note-code-constant (* (1- (sb!c:sc-offset-offset sc-offs))
+                                     sb!vm:n-word-bytes)
+                                  dstate)
+              (emit-note (get-sc-name sc-offs))))))
     (incf (dstate-next-offs dstate)
           adjust)))
