@@ -71,8 +71,10 @@
 (defmethod print-object ((method standard-method) stream)
   (if (slot-boundp method '%generic-function)
       (print-unreadable-object (method stream :type t :identity t)
-        (let ((generic-function (method-generic-function method)))
-          (format stream "~S ~{~S ~}~:S"
+        (let ((generic-function (method-generic-function method))
+              (*print-length* 50))
+          (format stream "~:[~*~;~/sb-impl::print-symbol-with-prefix/ ~]~{~S ~}~:S"
+                  generic-function
                   (and generic-function
                        (generic-function-name generic-function))
                   (method-qualifiers method)
@@ -85,7 +87,7 @@
   (if (slot-boundp method '%generic-function)
       (print-unreadable-object (method stream :type t :identity t)
         (let ((generic-function (method-generic-function method)))
-          (format stream "~S, slot:~S, ~:S"
+          (format stream "~/sb-impl::print-symbol-with-prefix/, slot:~S, ~:S"
                   (and generic-function
                        (generic-function-name generic-function))
                   (accessor-method-slot-name method)
@@ -108,9 +110,8 @@
          (let ((name (slot-value instance 'name)))
            (print-unreadable-object
                (instance stream :type t :identity (not properly-named-p))
-             (if extra-p
-                 (format stream "~S ~:S" name extra)
-                 (prin1 name stream)))))
+             (format stream "~/sb-impl::print-symbol-with-prefix/~:[~:; ~:S~]"
+                     name extra-p extra))))
         ((not extra-p) ; case (2): empty body to avoid an extra space
          (print-unreadable-object (instance stream :type t :identity t)))
         (t ; case (3). no name, but extra data - show #<unbound slot> and data
@@ -123,7 +124,7 @@
       (let* ((name (class-name class))
              (proper-p (and (symbolp name) (eq (find-class name nil) class))))
         (print-unreadable-object (class stream :type t :identity (not proper-p))
-          (prin1 name stream)))
+          (print-symbol-with-prefix stream name)))
       ;; "#<CLASS #<unbound slot> {122D1141}>" is ugly. Don't show that.
       (print-unreadable-object (class stream :type t :identity t))))
 
@@ -169,3 +170,9 @@
 (defmethod print-object ((obj class-precedence-description) stream)
   (print-unreadable-object (obj stream :type t)
     (format stream "~D" (cpd-count obj))))
+
+(defmethod print-object ((self eql-specializer) stream)
+  (let ((have-obj (slot-boundp self 'object)))
+    (print-unreadable-object (self stream :type t :identity (not have-obj))
+      (when have-obj
+        (write (slot-value self 'object) :stream stream)))))
