@@ -306,7 +306,14 @@
     (test ((a t)) (not a) "The assertion (NOT A) failed with A = T.")
     (test () (not t) "The assertion (NOT T) failed.")
     (test ((a -1)) (plusp (signum a))
-          "The assertion (PLUSP (SIGNUM A)) failed with (SIGNUM A) = -1.")))
+          "The assertion (PLUSP (SIGNUM A)) failed with (SIGNUM A) = -1.")
+    ;; Same for local functions.
+    (flet ((my-not (x) (not x))
+           (my-plusp (x) (plusp x)))
+      (test ((a t)) (my-not a) "The assertion (MY-NOT A) failed with A = T.")
+      (test () (my-not t) "The assertion (MY-NOT T) failed.")
+      (test ((a -1)) (my-plusp (signum a))
+            "The assertion (MY-PLUSP (SIGNUM A)) failed with (SIGNUM A) = -1."))))
 
 (with-test (:name (find-restart :recheck-conditions-and-tests :bug-774410))
   (let ((activep t))
@@ -393,3 +400,91 @@
                 (with-testing-restart (:condition-restart-p nil)
                   (invoke-restart-interactively
                    (find-restart 'testing-restart (make-condition 'condition))))))))
+
+(defun case-failure-example (x) (etypecase x (function 1) (symbol 2)))
+;; The :report method should not print "wanted one of #'SYMBOL"
+(with-test (:name :case-failure-report-pprint-silliness)
+  (handler-case (case-failure-example 3)
+    (condition (c)
+      (let ((str (write-to-string c :escape nil :pretty t)))
+        (assert (not (search "#'SYMBOL" str)))))))
+
+(define-condition a-condition-with-a-classy-slot ()
+  ((a :allocation :class :initform 'foo)))
+
+(defvar *a-classy-classoid*
+  (sb-kernel:find-classoid 'a-condition-with-a-classy-slot))
+;; precondition to the test
+(assert (= (length (sb-kernel::condition-classoid-class-slots
+                    *a-classy-classoid*))
+           1))
+
+(define-condition a-condition-with-a-classy-slot ()
+  ((b :allocation :class :initform 'baz)
+   (a :allocation :class :initform 'foo)))
+
+(with-test (:name :condition-classoid-redef-with-class-slot)
+  (assert (= (length (sb-kernel::condition-classoid-class-slots
+                      *a-classy-classoid*))
+             2)))
+
+(defun some-silly-handler (x) (declare (ignore x)))
+
+;; Note, you should exercise extreme caution when printing *HANDLER-CLUSTERS*
+;; from the REPL or you will likely confuse yourself.
+;; At any point, the clusters are composed of a chain of dx conses.
+;; One way to print them is like this:
+;;  (funcall (lambda () (print sb-kernel::*handler-clusters*) nil))
+
+(with-test (:name :handler-bind-expansion-variations)
+
+  ;; exercise all 3 possibilities for the TEST of a clause
+  (handler-bind ((error #'print))
+    ;; first cluster, first binding, first part
+    (let ((test (caaar sb-kernel:*handler-clusters*)))
+      (assert (eq test (sb-kernel:find-classoid-cell 'error)))))
+  (locally (declare (muffle-conditions style-warning))
+    (handler-bind (((satisfies some-silly-fun) #'print))
+      (let ((test (caaar sb-kernel:*handler-clusters*)))
+        (assert (eq test (sb-int:find-fdefn 'some-silly-fun))))))
+  (handler-bind (((or warning error) #'print))
+    (let ((test (caaar sb-kernel:*handler-clusters*)))
+      (assert (functionp test))))
+
+  ;; exercise all 3 possibilities for the HANDLER of a clause
+
+  (flet ((frob (x) (print x)))
+    (handler-bind ((warning
+                    (lambda (c) (frob (frob c)))))
+      ;; first cluster, first binding, second part
+      (let ((fn (cdaar sb-kernel:*handler-clusters*)))
+        (assert (functionp fn)))))
+
+  (handler-bind ((warning (eval ''muffle-warning)))
+    (let ((fn (cdaar sb-kernel:*handler-clusters*)))
+      ;; the function is stored as a symbol
+      (assert (eq fn 'muffle-warning))))
+
+  (handler-bind ((warning 'some-silly-handler))
+    (let ((fn (cdaar sb-kernel:*handler-clusters*)))
+      ;; the function is stored as an fdefn
+      (assert (typep fn 'sb-kernel::fdefn))))
+
+  (handler-bind ((warning 'muffle-warning))
+    (let ((fn (cdaar sb-kernel:*handler-clusters*)))
+      ;; the function is stored directly because it's a builtin.
+      (assert (eq fn (symbol-function 'muffle-warning))))))
+
+;; Oddly enough, at least one other Lisp considers this example
+;; *not* to fail, quite explicitly - the handler function in the
+;; handler-cluster is a lambda that returns a function that is
+;; the real handler function.
+(defun this-should-fail ()
+  (declare (muffle-conditions style-warning))
+  (handler-bind ((condition #'some-nonexistent-handler))
+    (random 100)))
+
+(with-test (:name :handler-bind-evals-handlers-immediately)
+  (assert-error (this-should-fail))
+  (defun some-nonexistent-handler (x) x)
+  (assert (integerp (this-should-fail)))) ; but not now it shouldn't
